/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.model.flowproperty;

import java.io.Serializable;

import javax.persistence.AssociationOverride;
import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

import de.fzk.iai.ilcd.service.model.IFlowPropertyVO;
import de.fzk.iai.ilcd.service.model.flowproperty.IUnitGroupType;
import de.iai.ilcd.model.common.DataSet;
import de.iai.ilcd.model.common.GlobalReference;
import de.iai.ilcd.model.common.MultiLanguageString;
import de.iai.ilcd.model.unitgroup.UnitGroup;

/**
 * 
 * @author clemens.duepmeier
 */
@Entity
@Table(uniqueConstraints = @UniqueConstraint(columnNames = { "UUID", "MAJORVERSION", "MINORVERSION", "SUBMINORVERSION" }))
public class FlowProperty extends DataSet implements Serializable, IFlowPropertyVO {

	/**
	 * Serialization ID
	 */
	private static final long serialVersionUID = 7473119545025737038L;

	@Embedded
	@AttributeOverride(name = "defaultValue", column = @Column(name = "synonyms"))
	@AssociationOverride(name = "lStringMap", joinTable = @JoinTable(name = "flowproperty_synonyms", joinColumns = @JoinColumn(name = "flowproperty_id", referencedColumnName = "ID")))
	protected MultiLanguageString synonyms = new MultiLanguageString();

	@ManyToOne(cascade = CascadeType.ALL)
	protected GlobalReference referenceToUnitGroup;

	@ManyToOne(fetch = FetchType.LAZY)
	protected UnitGroup unitGroup;

	@Override
	public MultiLanguageString getSynonyms() {
		return synonyms;
	}

	public void setSynonyms(MultiLanguageString synonyms) {
		this.synonyms = synonyms;
	}

	public GlobalReference getReferenceToUnitGroup() {
		return this.referenceToUnitGroup;
	}

	public void setReferenceToUnitGroup(GlobalReference referenceToUnitGroup) {
		this.referenceToUnitGroup = referenceToUnitGroup;
	}

	public UnitGroup getUnitGroup() {
		return unitGroup;
	}

	public void setUnitGroup(UnitGroup unitGroup) {
		this.unitGroup = unitGroup;
	}

	public MultiLanguageString getUnitGroupName() {
		MultiLanguageString unitGroupName = null;

		if (this.getUnitGroup() != null) {
			unitGroupName = getUnitGroup().getName();
		} else if (this.getReferenceToUnitGroup() != null) {
			unitGroupName = new MultiLanguageString(this.getReferenceToUnitGroup().getShortDescription());
		}

		return unitGroupName;
	}

	public String getDefaultUnit() {
		if (this.getUnitGroup() != null)
			return this.getUnitGroup().getReferenceUnit().getName();
		// OK, we can't find the unit group, let's use the GlobalReference
		if (referenceToUnitGroup != null)
			return referenceToUnitGroup.getShortDescription().getDefaultValue();
		// No chance, for meaningful information
		return null;
	}

	@Override
	public IUnitGroupType getUnitGroupDetails() {
		return new UnitGroupDetails(unitGroup, referenceToUnitGroup);
	}

	@Override
	public String toString() {
		return "de.iai.ilcd.model.flowproperty.FlowProperty[id=" + id + "]";
	}

}
