/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.model.security;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.ElementCollection;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;

/**
 * 
 * @author clemens.duepmeier
 */
@Entity
public class User implements Serializable {
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private Long id;

	@Column(unique = true)
	private String userName;

	@Column(name="PASSWORD_HASH")
	private String passwordHash;

	@Column(name="PASSWORD_HASH_SALT")
	private String passwordHashSalt;

	private String firstName;

	private String lastName;

	private String title;

	@Enumerated(EnumType.STRING)
	private Gender gender = null;

	private String email;

	private String organisation;

	private String organisationUnit;

	private String registrationKey;

	@ManyToOne(fetch = FetchType.EAGER)
	private IndustrialSector sector;

	@Embedded
	private Address address = new Address();

	@Enumerated(EnumType.STRING)
	@ElementCollection
	private List<SecurityRole> roles;

	@ManyToMany(mappedBy = "users", fetch = FetchType.EAGER, cascade = { CascadeType.PERSIST, CascadeType.MERGE, CascadeType.REFRESH, CascadeType.DETACH }) // Does not contain CascadeType.REMOVE
	private List<UserGroup> groups;

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public Address getAddress() {
		return address;
	}

	public void setAddress(Address address) {
		this.address = address;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public String getFirstName() {
		return firstName;
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	public Gender getGender() {
		return gender;
	}

	public void setGender(Gender gender) {
		this.gender = gender;
	}

	public String getLastName() {
		return lastName;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	public String getOrganisation() {
		return organisation;
	}

	public void setOrganisation(String organisation) {
		this.organisation = organisation;
	}

	public String getOrganisationUnit() {
		return organisationUnit;
	}

	public void setOrganisationUnit(String organisationUnit) {
		this.organisationUnit = organisationUnit;
	}

	public String getPasswordHash() {
		return this.passwordHash;
	}
	
	public void setPasswordHash(String passwordHash) {
		this.passwordHash = passwordHash;
	}
	
	public String getPasswordHashSalt() {
		return this.passwordHashSalt;
	}
	
	public void setPasswordHashSalt(String passwordHashSalt) {
		this.passwordHashSalt = passwordHashSalt;
	}

	public IndustrialSector getSector() {
		return sector;
	}

	public void setSector(IndustrialSector sector) {
		this.sector = sector;
	}

	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public String getUserName() {
		return userName;
	}

	public void setUserName(String userName) {
		this.userName = userName;
	}

	public String getRegistrationKey() {
		return registrationKey;
	}

	public void setRegistrationKey(String registrationKey) {
		this.registrationKey = registrationKey;
	}

	public List<SecurityRole> getRoles() {
		return roles;
	}

	protected void setRoles(List<SecurityRole> roles) {
		this.roles = roles;
	}

	public void addRole(SecurityRole role) {
		if (!roles.contains(role))
			roles.add(role);
	}

	public void deleteRole(SecurityRole role) {
		if (roles.contains(role))
			roles.remove(role);
	}

	public boolean hasRole(SecurityRole role) {
		if (roles.contains(role))
			return true;
		// now check groups
		for (UserGroup group : groups) {
			if (group.hasRole(role))
				return true;
		}

		return false;
	}

	public List<UserGroup> getGroups() {
		return groups;
	}

	protected void setGroups(List<UserGroup> groups) {
		this.groups = groups;
	}

	public void addToGroup(UserGroup group) {
		if (!groups.contains(group)) {
			group.addUser(this);
			groups.add(group);
		}
	}

	public void removeFromGroup(UserGroup group) {
		if (groups.contains(group)) {
			groups.remove(group);
		}
	}

	public void removeFromAllGroups() {
		List<UserGroup> allGroups = new ArrayList<UserGroup>();
		allGroups.addAll(groups);
		for (UserGroup group : allGroups) {
			group.deleteUser(this); // this removes the group also locally
		}
	}

	@Override
	public int hashCode() {
		int hash = 0;
		hash += (userName != null ? userName.hashCode() : 0);
		return hash;
	}

	@Override
	public boolean equals(Object object) {
		// TODO: Warning - this method won't work in the case the id fields are not set
		if (!(object instanceof User)) {
			return false;
		}
		User other = (User) object;
		if (userName != null && other.getUserName() != null && userName.equals(other.getUserName())) {
			return true;
		}
		return false;
	}

	@Override
	public String toString() {
		return "de.iai.ilcd.model.security.User[userName=" + userName + "]";
	}

}
