/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.model.unitgroup;

import java.io.Serializable;

import javax.persistence.AssociationOverride;
import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;

import de.iai.ilcd.model.common.MultiLanguageText;

/**
 * 
 * @author clemens.duepmeier
 */
@Entity
public class Unit implements Serializable {
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private Long id;

	protected int internalId;

	@Column(name = "unitname")
	protected String name;

	@Embedded
	@AttributeOverride(name = "defaultValue", column = @Column(name = "description"))
	@AssociationOverride(name = "lStringMap", joinTable = @JoinTable(name = "unit_description", joinColumns = @JoinColumn(name = "unit_id", referencedColumnName = "ID")))
	protected MultiLanguageText description;

	protected double meanValue;

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public MultiLanguageText getDescription() {
		return description;
	}

	public void setDescription(MultiLanguageText description) {
		this.description = description;
	}

	public int getInternalId() {
		return internalId;
	}

	public void setInternalId(int internalId) {
		this.internalId = internalId;
	}

	public double getMeanValue() {
		return meanValue;
	}

	public void setMeanValue(double meanValue) {
		this.meanValue = meanValue;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	@Override
	public int hashCode() {

		return name.hashCode();
	}

	@Override
	public boolean equals(Object object) {
		// TODO: Warning - this method won't work in the case the id fields are not set
		if (!(object instanceof Unit)) {
			return false;
		}
		Unit other = (Unit) object;

		if ((this.id != null && !this.id.equals(other.id))) {
			return false;
		}
		if (this.name.equals(other.getName()))
			return true;
		else
			return true;
	}

	@Override
	public String toString() {
		return "de.iai.ilcd.model.unitgroup.Unit[name=" + name + "]";
	}

}
