/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.security;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.shiro.authz.AuthorizationInfo;
import org.apache.shiro.authz.SimpleAuthorizationInfo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.iai.ilcd.model.dao.UserDao;
import de.iai.ilcd.model.security.SecurityRole;
import de.iai.ilcd.model.security.User;
import de.iai.ilcd.model.security.UserGroup;

/**
 * 
 * @author clemens.duepmeier
 */
public class IlcdAuthorizationInfo extends SimpleAuthorizationInfo implements AuthorizationInfo {

	private String userName;

	private UserDao dao = new UserDao();

	Logger logger = LoggerFactory.getLogger(IlcdAuthorizationInfo.class);

	public IlcdAuthorizationInfo(String userName) {
		this.userName = userName;

	}

	@Override
	public Set<String> getRoles() {
		Set<String> roleNames = new HashSet<String>();
		// logger.info("getRoles was called");
		User user = null;
		try {
			user = dao.getUser(userName);
			if (user != null) {
				// first take users own userRoles
				List<SecurityRole> userRoles = user.getRoles();

				for (SecurityRole role : userRoles) {
					roleNames.add(role.getValue());
					logger.info("Add role name {}", role.getValue());
				}
				// and add group specific userRoles
				for (UserGroup group : user.getGroups()) {
					List<SecurityRole> groupRoles = group.getRoles();
					for (SecurityRole groupRole : groupRoles) {
						if (!roleNames.contains(groupRole.getValue())) {
							roleNames.add(groupRole.getValue());
							logger.info("Add group role name {}", groupRole.getValue());
						}
					}
				}
			}
		} catch (Exception e) {
			// do nothing; we return an empty list then
		}
		return roleNames;
	}

}
