/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.webgui.controller.admin;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;

import org.apache.velocity.tools.generic.ValueParser;
import org.primefaces.component.datatable.DataTable;
import org.primefaces.model.LazyDataModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.iai.ilcd.model.contact.Contact;
import de.iai.ilcd.model.dao.ContactDao;
import de.iai.ilcd.model.dao.DeleteDataSetException;

/**
 * 
 * @author clemens.duepmeier
 */
@ManagedBean()
@ViewScoped
public class ContactListHandler implements Serializable {

	private static Logger logger = LoggerFactory.getLogger(ContactListHandler.class);

	private LazyDataModel<Contact> lazyModel;

	private Contact[] selectedContacts;

	private ValueParser params;

	public ContactListHandler() {
		lazyModel = new ContactDataModel();

		ContactDao dao = new ContactDao();
		params = new ValueParser();
		long rowCount = dao.searchResultCount(params,false);
		lazyModel.setRowCount((int) rowCount);
	}

	public LazyDataModel<Contact> getLazyModel() {
		return lazyModel;
	}

	public void setLazyModel(LazyDataModel<Contact> lazyModel) {
		this.lazyModel = lazyModel;
	}

	public Contact[] getSelectedContacts() {
		return selectedContacts;
	}

	public void setSelectedContacts(Contact[] selectedContacts) {
		this.selectedContacts = selectedContacts;
	}

	public void deleteSelected() {
		if (selectedContacts == null) {
			return;
		}

		ContactDao cDao = new ContactDao();

		for (Contact contact : selectedContacts) {
			try {
				cDao.remove(contact);
			} catch (DeleteDataSetException ex) {
				FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_ERROR, "Could not remove contact "
						+ contact.getName().getDefaultValue() + "\n", "");
				FacesContext.getCurrentInstance().addMessage("error", facesMsg);
			}
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_INFO, "Removed contact "
					+ contact.getName().getDefaultValue() + "\n", "");
			FacesContext.getCurrentInstance().addMessage("info", facesMsg);
		}
		// set selected contacts to null
		selectedContacts = null;

		int rowCount = (int) cDao.searchResultCount(params,false);
		lazyModel.setRowCount(rowCount);
		DataTable dataTable = (DataTable) FacesContext.getCurrentInstance().getViewRoot()
				.findComponent("tableForm:contactTable");
		if (dataTable != null)
			dataTable.loadLazyData();

		return;
	}

	private class ContactDataModel extends LazyDataModel<Contact> {
		// we have to overload the load()-method of the lazy data model

		@Override
		public List<Contact> load(int first, int pageSize, String sortField, boolean sortOrder,
				Map<String, String> filters) {
			logger.info("Loading the lazy contact data between {0} and {1}", new Object[] { first, (first + pageSize) });

			// Sorting and Filtering information are not used for demo purposes just random dummy data is returned

			List<Contact> lazyContacts = new ArrayList<Contact>();
			// now let's fill it from the database
			ContactDao dao = new ContactDao();
			// ValueParser params = new ValueParser();
			lazyContacts = dao.search(Contact.class, params, first, pageSize, sortField, false);

			long rowCount = dao.searchResultCount(params,false);
			this.setRowCount((int) rowCount);
			return lazyContacts;
		}
	}
}
