package de.iai.ilcd.webgui.controller.admin;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import javax.faces.application.Resource;
import javax.faces.application.ResourceHandler;
import javax.faces.application.ResourceWrapper;
import javax.faces.context.FacesContext;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.fzk.iai.ilcd.zip.ILCDZip;
import de.iai.ilcd.configuration.ConfigurationService;
import de.iai.ilcd.webgui.controller.ui.DataExportHandler;

/**
 * Proxy Resource for the ILCD Service downloads
 */

public class DataExportResource extends ResourceWrapper {

	public static Logger logger = LoggerFactory.getLogger(de.iai.ilcd.webgui.controller.admin.DataExportResource.class);

	/**
	 * MIME type to set
	 */
	public final static String CONTENT_TYPE = "application/zip";

	/**
	 * Name of the resource
	 */
	public final static String RESOURCE_NAME = "dataexport";

	/**
	 * The local JSF resource path for the proxy object
	 */
	private String requestPath = null;

	/**
	 * Create the handler
	 */
	public DataExportResource() {

		logger.trace("instantiate DataExportResource");

		this.setLibraryName(DataExportResourceHandler.LIBRARY_NAME);
		this.setContentType(DataExportResource.CONTENT_TYPE);

		this.setResourceName(DataExportResource.RESOURCE_NAME);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public InputStream getInputStream() throws IOException {

//		logger.trace("getInputStream");

		DataExportHandler handler = new DataExportHandler();

		ILCDZip zip = handler.export();

		if (zip != null) {
			FileInputStream str = new FileInputStream(zip);
			return str;
		} else {
			return null;
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getRequestPath() {

		if (this.requestPath == null) {
			StringBuilder buf = new StringBuilder();
			buf.append(FacesContext.getCurrentInstance().getExternalContext().getRequestContextPath());
			buf.append(ResourceHandler.RESOURCE_IDENTIFIER);
			buf.append("/");
			buf.append(this.getResourceName());
			buf.append(".xhtml");
			buf.append("?ln=");
			buf.append(this.getLibraryName());
			this.requestPath = buf.toString();
		}
//		logger.trace("getRequestPath " + this.requestPath);
		return this.requestPath;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Map<String, String> getResponseHeaders() {
		Map<String, String> map = new HashMap<String, String>();
		map.put("Content-disposition", "attachment; filename=" + ConfigurationService.INSTANCE.getContextPath().replaceAll("/", "") + ".zip");
		return map;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public URL getURL() {
		return null;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean userAgentNeedsUpdate(FacesContext arg0) {
		return true;
	}

	@Override
	public Resource getWrapped() {
		// TODO Auto-generated method stub
		return null;
	}

}
