/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.webgui.controller.admin;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.RequestScoped;
import javax.faces.context.FacesContext;

import org.primefaces.model.DualListModel;

import de.iai.ilcd.model.dao.UserDao;
import de.iai.ilcd.model.dao.UserGroupDao;
import de.iai.ilcd.model.security.SecurityRole;
import de.iai.ilcd.model.security.User;
import de.iai.ilcd.model.security.UserGroup;

/**
 * 
 * @author clemens.duepmeier
 */
@ManagedBean
@RequestScoped
public class GroupHandler implements Serializable {

	UserGroup group = new UserGroup();

	List<String> selectedRoles1 = new ArrayList<String>();

	List<String> selectedRoles2 = new ArrayList<String>();

	List<String> roles1;

	List<String> roles2;

	DualListModel<String> dualUserList;

	/** Creates a new instance of UserHandler */
	public GroupHandler() {
		// initialize list of users
		UserDao userDao = new UserDao();
		List<User> allUsers = userDao.getUsers();
		List<String> source = new ArrayList<String>();
		List<String> target = new ArrayList<String>();
		for (User user : allUsers) {
			source.add(user.getUserName());
		}
		// check if the page was called with a request parameter specifying the current user
		String groupId = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap().get("groupId");
		if (groupId != null) {
			// the URL is called with a given groupId, so initialize a corresponding UserGroup object
			UserGroupDao groupDao = new UserGroupDao();
			try {
				UserGroup existingGroup = groupDao.getGroup(Long.parseLong(groupId));
				if (existingGroup != null)
					group = existingGroup;

				List<User> groupUsers = group.getUsers();
				for (User user : groupUsers) {
					String userName = user.getUserName();
					target.add(userName);
					source.remove(userName);
				}

				fillSelectedRoles();

			} catch (Exception e) {
				// we just do nothing; maybe wrong parameter?
			}
		}
		dualUserList = new DualListModel<String>(source, target);
	}

	public UserGroup getGroup() {
		return group;
	}

	public void setGroup(UserGroup group) {
		this.group = group;
	}

	public List<String> getSelectedRoles1() {
		return selectedRoles1;
	}

	public void setSelectedRoles1(List<String> selectedRoles1) {
		this.selectedRoles1 = selectedRoles1;
	}

	public List<String> getSelectedRoles2() {
		return selectedRoles2;
	}

	public void setSelectedRoles2(List<String> selectedRoles2) {
		this.selectedRoles2 = selectedRoles2;
	}

	public DualListModel<String> getDualUserList() {
		return dualUserList;
	}

	public void setDualUserList(DualListModel<String> dualUserList) {
		this.dualUserList = dualUserList;
	}

	public void createGroup() {
		UserGroupDao dao = new UserGroupDao();
		UserDao userDao = new UserDao();

		if (group.getGroupName() == null) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN, "No group name provided", "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
			return;
		}

		UserGroup existingGroup = dao.getGroup(group.getGroupName());
		if (existingGroup != null) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN,
					"A group with this name already exists in the system", "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
		}

		// transfer selected users to group object
		for (String userName : dualUserList.getTarget()) {
			// group.getUsers().clear();
			User user = userDao.getUser(userName);
			group.addUser(user);
		}

		// set roles on the group object
		// group.getRoles().clear();
		for (String roleName : selectedRoles1) {
			group.addRole(SecurityRole.fromValue(roleName));
		}
		for (String roleName : selectedRoles2) {
			group.addRole(SecurityRole.fromValue(roleName));
		}

		try {
			dao.persist(group);
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_INFO, "The group has been created", "");
			FacesContext.getCurrentInstance().addMessage("info", facesMsg);
		} catch (Exception e) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN,
					"There were some system errors while creating the group; please try again", "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
		}

	}

	public void changeGroup() {
		UserGroupDao dao = new UserGroupDao();
		UserDao userDao = new UserDao();

		if (group.getGroupName() == null) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN, "No group name provided", "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
			return;
		}

		UserGroup existingGroup = dao.getGroup(group.getGroupName());
		group = existingGroup;
		if (existingGroup == null) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN, "A group with the name "
					+ group.getGroupName() + " does not exist", "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
			return;
		}

		// synchronize users
		// clear old group
		List<User> tempList = new ArrayList<User>();
		tempList.addAll(group.getUsers());
		// we need to do it in that way so that the users side is synchronized too, i.e. remove group from user object
		for (User user : tempList) {
			group.deleteUser(user);
		}
		// group.getUsers().clear();
		// now we can add the users again
		for (String userName : dualUserList.getTarget()) {
			User user = userDao.getUser(userName);
			group.addUser(user);
		}

		// set roles on the group object
		group.getRoles().clear();
		for (String roleName : selectedRoles1) {
			group.addRole(SecurityRole.fromValue(roleName));
		}
		for (String roleName : selectedRoles2) {
			group.addRole(SecurityRole.fromValue(roleName));
		}

		try {
			dao.merge(group);
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_INFO,
					"Changes to the group have been saved", "");
			FacesContext.getCurrentInstance().addMessage("info", facesMsg);
		} catch (Exception e) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN,
					"There were some system errors while saving changes; please try again", "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
		}
	}

	private void fillSelectedRoles() {
		List<SecurityRole> roles = group.getRoles();
		// delete current information
		selectedRoles1.clear();
		selectedRoles2.clear();

		for (SecurityRole role : roles) {
			switch (role) {
			case READ_ROLE:
			case EXPORT_ROLE:
			case CHECKOUT_ROLE:
			case CHECKIN_ROLE:
				selectedRoles1.add(role.getValue());
				break;
			case ADMIN_ROLE:
			case DELETE_ROLE:
			case MANAGE_USER_ROLE:
			case RELEASE_ROLE:
				selectedRoles2.add(role.getValue());
				break;
			default:
			}
		}
	}
}
