/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.webgui.controller.admin;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;

import org.apache.velocity.tools.generic.ValueParser;
import org.primefaces.component.datatable.DataTable;
import org.primefaces.model.LazyDataModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.iai.ilcd.model.dao.DeleteDataSetException;
import de.iai.ilcd.model.dao.SourceDao;
import de.iai.ilcd.model.source.Source;

/**
 * 
 * @author clemens.duepmeier
 */
@ManagedBean()
@ViewScoped
public class SourceListHandler implements Serializable {

	private static Logger logger = LoggerFactory.getLogger(SourceListHandler.class);

	private LazyDataModel<Source> lazyModel;

	private Source[] selectedSources;

	private ValueParser params;

	public SourceListHandler() {
		lazyModel = new SourceDataModel();

		SourceDao dao = new SourceDao();
		params = new ValueParser();
		long rowCount = dao.searchResultCount(params,false);
		lazyModel.setRowCount((int) rowCount);
	}

	public LazyDataModel<Source> getLazyModel() {
		return lazyModel;
	}

	public void setLazyModel(LazyDataModel<Source> lazyModel) {
		this.lazyModel = lazyModel;
	}

	public Source[] getSelectedSources() {
		return selectedSources;
	}

	public void setSelectedSources(Source[] selectedSources) {
		this.selectedSources = selectedSources;
	}

	public void deleteSelected() {
		if (selectedSources == null) {
			return;
		}

		SourceDao sDao = new SourceDao();

		for (Source source : selectedSources) {
			try {
				sDao.remove(source);
			} catch (DeleteDataSetException ex) {
				FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_ERROR, "Could not remove source "
						+ source.getName().getDefaultValue() + "\n", "");
				FacesContext.getCurrentInstance().addMessage("error", facesMsg);
			}
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_INFO, "Removed source "
					+ source.getName().getDefaultValue() + "\n", "");
			FacesContext.getCurrentInstance().addMessage("info", facesMsg);
		}

		selectedSources = null;

		int rowCount = (int) sDao.searchResultCount(params,false);
		lazyModel.setRowCount(rowCount);
		DataTable dataTable = (DataTable) FacesContext.getCurrentInstance().getViewRoot()
				.findComponent("tableForm:sourceTable");
		if (dataTable != null)
			dataTable.loadLazyData();

		return;
	}

	private class SourceDataModel extends LazyDataModel<Source> {
		// we have to overload the load()-method of the lazy data model

		@Override
		public List<Source> load(int first, int pageSize, String sortField, boolean sortOrder,
				Map<String, String> filters) {
			logger.info("Loading the lazy source data between {0} and {1}", new Object[] { first, (first + pageSize) });

			// Sorting and Filtering information are not used for demo purposes just random dummy data is returned

			List<Source> lazySources = new ArrayList<Source>();
			// now let's fill it from the database
			SourceDao dao = new SourceDao();
			// ValueParser params = new ValueParser();
			lazySources = dao.search(Source.class, params, first, pageSize, sortField,false);

			long rowCount = dao.searchResultCount(params,false);
			this.setRowCount((int) rowCount);
			return lazySources;
		}
	}
}
