/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/
package de.iai.ilcd.webgui.controller.ui;

import javax.faces.bean.ManagedBean;

import de.fzk.iai.ilcd.service.model.IProcessVO;
import de.fzk.iai.ilcd.service.model.enums.LCIMethodApproachesValue;
import de.fzk.iai.ilcd.service.model.enums.TypeOfFlowValue;
import de.fzk.iai.ilcd.service.model.process.IComplianceSystem;
import de.iai.ilcd.model.dao.ProcessDao;
import de.iai.ilcd.model.flow.Flow;
import de.iai.ilcd.model.process.Exchange;
import de.iai.ilcd.model.process.ExchangeDirection;
import de.iai.ilcd.model.process.Process;
import java.util.ArrayList;
import java.util.List;

/**
 * Backing bean for process detail view
 */
@ManagedBean
public class ProcessHandler extends AbstractDataSetHandler<IProcessVO, Process, ProcessDao> {

    /**
     * Serialization ID
     */
    private static final long serialVersionUID = 6907352892423743765L;

    /**
     * List with input product flows
     */
    private List<Flow> inputProducts;
    
    /**
     * List with co product flows
     */
    private List<Flow> coProducts;
    
    /**
     * Initialize the handler
     */
    public ProcessHandler() {
        super(new ProcessDao(), "processes");
    }
    
    /**
     * Convenience method, delegates to {@link #getDataSet()}
     * 
     * @return process
     */
    public IProcessVO getProcess() {
        return this.getDataSet();
    }

    /**
     * We need this as workaround because the Service API delivers a set but we need a List in the template
     * @return 
     */
    public List<LCIMethodApproachesValue> getApproaches() {
        List<LCIMethodApproachesValue> approaches = new ArrayList<LCIMethodApproachesValue>();

        if (this.getDataSet() != null) {
            approaches = new ArrayList<LCIMethodApproachesValue>(this.getDataSet().getLCIMethodInformation().getApproaches());
        }

        return approaches;
    }
    
    
    /**
     * We need this too as workaround to geth the ComplianceSystems as List
     * @return 
     */
    public List<IComplianceSystem> getComplianceSystems() {
        List<IComplianceSystem> complianceSystems = new ArrayList<IComplianceSystem>();

        if (this.getDataSet() != null) {
            complianceSystems = new ArrayList<IComplianceSystem>(this.getDataSet().getComplianceSystems());
        }

        return complianceSystems;
    }

    /**
     * Get the input product flows
     * @return input product flows
     */
    public List<Flow> getInputProducts() {
		return this.inputProducts;
	}
    
    /**
     * Get the co-product flows
     * @return co-product flows
     */
    public List<Flow> getCoProducts() {
		return this.coProducts;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    protected void datasetLoaded(Process p) {
    	// Load input products
    	List<Exchange> tmp = p.getExchanges( ExchangeDirection.INPUT.name() );
    	this.inputProducts = new ArrayList<Flow>();
    	for( Exchange e : tmp ){
    		if ( e.getFlow() != null && TypeOfFlowValue.PRODUCT_FLOW.equals(e.getFlow().getType()) ){
    			this.inputProducts.add(e.getFlow());
    		}
    	}
    	
    	// Load co-products
    	tmp = p.getExchanges( ExchangeDirection.OUTPUT.name() );
    	this.coProducts = new ArrayList<Flow>();
    	// named loop (due to continue from inner loop)
    	coProdLoop: for( Exchange e : tmp ){
    		if ( e.getFlow() != null && TypeOfFlowValue.PRODUCT_FLOW.equals(e.getFlow().getType()) ){
    			// check if this exchange flow is contained in the reference exchange list
    			Long id = e.getFlow().getId();
    			for( Exchange refEx : p.getReferenceExchanges() ){
    				if ( refEx.getFlow() != null && id.equals( refEx.getFlow().getId()) ){
    					// trigger next loop from !! outer !! loop
    					continue coProdLoop;
    				}
    			}
    			// flow was not in reference exchange list
    			this.coProducts.add(e.getFlow());
    		}
    	}    	
    }
}
